window.Gleam.initSearch([{"doc":"rememo","title":"rememo","content":"# rememo\n\n[![Package Version](https://img.shields.io/hexpm/v/rememo)](https://hex.pm/packages/rememo)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/rememo/)\n\nThis is a basic [memoization](https://en.wikipedia.org/wiki/Memoization) method for Gleam on the Erlang target, for caching and reusing the results of function calls when possible.  \n\nAdding to and reading from the memoization cache incurs some overhead, but in situations that call for it, like dynamic programming problems where the same basic subproblem recurs hundreds or thousands of times, the tradeoff is worthwhile.  Always benchmark your code when in doubt.\n\n```sh\ngleam add rememo\n```\n\n```gleam\nimport rememo/ets/memo // This is the recommended implementation to use\nimport gleam/io\n\npub fn main() {\n  // Start the actor that holds the cached values\n  // for the duration of this block\n  use cache &lt;- memo.create()\n  fib(300, cache)\n  |&gt; io.debug\n}\n\nfn fib(n, cache) {\n  // Check if a value exists for the key n\n  // Use it if it exists, update the cache if it doesn&#39;t\n  use &lt;- memo.memoize(cache, n)\n  case n {\n    1 | 2 -&gt; 1\n    n -&gt; fib(n - 1, cache) + fib(n - 2, cache)\n  }\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/rememo&gt;.\n\n## Contributing and development\n\nSuggestions and pull requests are welcome.\n","url":"index.html"},{"doc":"rememo/ets/memo","title":"create","content":"pub fn create(apply fun: fn(Set(a, b)) -&gt; c) -&gt; c\n Makes a reference to a new ETS table that stores the memoization cache and passes this\n reference to a callback function containing the function you want to memoize.  \n The table is private and has a randomly-generated name. The table is dropped after `create` returns.\n \n Using `create` can be done easily with a `use` expression:\n \n use cache &lt;- create()\n f(a, b, c, cache)\n \n \nSynonyms:\nrememo/ets/memo.create\nrememo/ets/memo create","url":"rememo/ets/memo.html#create"},{"doc":"rememo/ets/memo","title":"get","content":"pub fn get(from cache: Set(a, b), fetch key: a) -&gt; Result(b, Nil)\n Manually look up a value from the memoization cache for a given key.\nSynonyms:\nrememo/ets/memo.get\nrememo/ets/memo get","url":"rememo/ets/memo.html#get"},{"doc":"rememo/ets/memo","title":"memoize","content":"pub fn memoize(\n  with cache: Set(a, b),\n  this key: a,\n  apply fun: fn() -&gt; b,\n) -&gt; b\n Look up the value associated with the given key in the memoization cache,\n and return it if it exists.  If it doesn&#39;t exist, evaluate the callback function\n and update the cache with the value it returns.\n \n The key can be of any data type, as long as the value the function returns is unique to that particular key.\n \n This works well with a `use` expression:\n \n fn f(a, b, c, cache) {\n   use &lt;- memoize(cache, #(a, b, c))\n   // function body goes here\n }\n \n \nSynonyms:\nrememo/ets/memo.memoize\nrememo/ets/memo memoize","url":"rememo/ets/memo.html#memoize"},{"doc":"rememo/ets/memo","title":"set","content":"pub fn set(\n  in cache: Set(a, b),\n  for key: a,\n  insert value: b,\n) -&gt; Nil\n Manually add a key-value pair to the memoization cache.\nSynonyms:\nrememo/ets/memo.set\nrememo/ets/memo set","url":"rememo/ets/memo.html#set"},{"doc":"rememo/ets/memo","title":"rememo/ets/memo","content":" This is the memoization implementation that uses [Erlang Term Storage](https://www.erlang.org/doc/man/ets.html) (ETS).  \n This is the faster (and newer) of the two implementations.","url":"rememo/ets/memo.html"},{"doc":"rememo/otp/memo","title":"Cache","content":"pub opaque type Cache(k, v)\n\n\nSynonyms:\nrememo/otp/memo.Cache\nrememo/otp/memo Cache","url":"rememo/otp/memo.html#Cache"},{"doc":"rememo/otp/memo","title":"create","content":"pub fn create(apply fun: fn(Cache(a, b)) -&gt; c) -&gt; c\n Start an actor that holds a memoization cache, and passes the\n reference to a callback function containing the function you want to memoize.  \n The actor is shut down after `create` returns, and the cache is erased with it.\n \n Using `create` can be done easily with a `use` expression:\n \n use cache &lt;- create()\n f(a, b, c, cache)\n \n \nSynonyms:\nrememo/otp/memo.create\nrememo/otp/memo create","url":"rememo/otp/memo.html#create"},{"doc":"rememo/otp/memo","title":"get","content":"pub fn get(\n  from cache: Cache(a, b),\n  fetch key: a,\n) -&gt; Result(b, Nil)\n Manually look up a value from the memoization cache for a given key.\nSynonyms:\nrememo/otp/memo.get\nrememo/otp/memo get","url":"rememo/otp/memo.html#get"},{"doc":"rememo/otp/memo","title":"memoize","content":"pub fn memoize(\n  with cache: Cache(a, b),\n  this key: a,\n  apply fun: fn() -&gt; b,\n) -&gt; b\n Look up the value associated with the given key in the memoization cache,\n and return it if it exists.  If it doesn&#39;t exist, evaluate the callback function\n and update the cache with the value it returns.\n \n The key can be of any data type, as long as the value the function returns is unique to that particular key.\n \n This works well with a `use` expression:\n \n fn f(a, b, c, cache) {\n   use &lt;- memoize(cache, #(a, b, c))\n   // function body goes here\n }\n \nSynonyms:\nrememo/otp/memo.memoize\nrememo/otp/memo memoize","url":"rememo/otp/memo.html#memoize"},{"doc":"rememo/otp/memo","title":"set","content":"pub fn set(\n  in cache: Cache(a, b),\n  for key: a,\n  insert value: b,\n) -&gt; Nil\n Manually add a key-value pair to the memoization cache.\nSynonyms:\nrememo/otp/memo.set\nrememo/otp/memo set","url":"rememo/otp/memo.html#set"},{"doc":"rememo/otp/memo","title":"rememo/otp/memo","content":" This is the memoization implementation that uses [`gleam/otp/actor`](https://www.erlang.org/doc/man/ets.html).  \n This is the slower (and original) of the two implementations.","url":"rememo/otp/memo.html"}]);